package {
	
	public class Selection {
		
		private var parent:Field = null;
		public var selected:Array = new Array();

		public function Selection( parent:Field ) {
			this.parent = parent;

			for( var x = 0; x < this.parent.size.x; x++ ) {
				this.selected[x] = new Array();
				for( var y = 0; y < this.parent.size.y; y++ ) {
					this.selected[x][y] = 0;
				}
			}
		}
		
		private function world2x( value:Number ) {
			return Math.round( value * (this.parent.size.x-1) );
		}

		private function world2y( value:Number ) {
			return Math.round( value * (this.parent.size.y-1) );
		}

		
		// Copy the source selection to our own
		public function copy( source:Array ) {
			for( var x = 0; x < this.parent.size.x; x++ ) {
				for( var y = 0; y < this.parent.size.y; y++ ) {
					this.selected[x][y] = source[x][y];
				}
			}
		}


		// Main working function - calls an Effect for each selection in the grid
		public function each( callback:Function, params:Object, tick:Number ) {
			for( var x = 0; x < this.parent.size.x; x++ ) {
				for( var y = 0; y < this.parent.size.y; y++ ) {
					if( this.selected[x][y] > 0 ) {
						callback( this.parent.dots[x][y], tick, x, y, this.selected[x][y], params );
					}
				}
			}

			return this;
		}

		
		// All ----------------------------------------------------------------------------------------------
		public function all( amount:Number = 1 ) {
			var result:Selection = new Selection( this.parent );

			for( var x = 0; x < this.parent.size.x; x++ ) {
				for( var y = 0; y < this.parent.size.y; y++ ) {
					result.selected[x][y] = amount;
				}
			}
			return result;
		}


		// Rect ---------------------------------------------------------------------------------------------
		public function rect( left, top, right, bottom, amount:Number = 1 ) {
			var result:Selection = new Selection( this.parent );
			result.copy( this.selected );
			left = this.world2x( left );
			top = this.world2y( top );
			right = this.world2x( right );
			bottom = this.world2y( bottom );

			for( var x = left; x < right+1; x++ ) {
				result.selected[x][top] = amount;
				result.selected[x][bottom] = amount;
			}

			for( var y = top+1; y < bottom; y++ ) {
				result.selected[left][y] = amount;
				result.selected[right][y] = amount;
			}

			return result;
		}


		// Circle -------------------------------------------------------------------------------------------
		public function circle( cx:Number, cy:Number, r:Number, amount:Number = 1 ) {
			var result:Selection = new Selection( this.parent );
			result.copy( this.selected );

			cx = this.world2x( cx );
			cy = this.world2y( cy );
			r = this.world2y( r );

			var x:Number = 0;
			var y:Number = r;
			var p:Number = ( 5 - r * 4 )/4;

			result.circleSegment( Math.floor(cx), Math.floor(cy), Math.floor(x), Math.floor(y), amount );
			while( x < y ) {
				x++;
				if (p < 0) {
					p += 2 * x + 1;
				} else {
					y--;
					p += 2 * ( x-y ) + 1;
				}
				result.circleSegment( Math.floor(cx), Math.floor(cy), Math.floor(x), Math.floor(y), amount );
			}

			return result;
		}

		private function circleSegment( cx:Number, cy:Number, x:Number, y:Number, amount:Number = 1 ) {
			if( x == 0 ) {
				this.selected[cx][cy + y] = amount;
				this.selected[cx][cy - y] = amount;
				this.selected[cx + y][cy] = amount;
				this.selected[cx - y][cy] = amount;
			} else
			if( x == y ) {
				this.selected[cx + x][cy + y] = amount;
				this.selected[cx - x][cy + y] = amount;
				this.selected[cx + x][cy - y] = amount;
				this.selected[cx - x][cy - y] = amount;
			} else
			if( x < y ) {
				this.selected[cx + x][cy + y] = amount;
				this.selected[cx - x][cy + y] = amount;
				this.selected[cx + x][cy - y] = amount;
				this.selected[cx - x][cy - y] = amount;
				this.selected[cx + y][cy + x] = amount;
				this.selected[cx - y][cy + x] = amount;
				this.selected[cx + y][cy - x] = amount;
				this.selected[cx - y][cy - x] = amount;
			}
		}


		// Fill ---------------------------------------------------------------------------------------------
		public function fill( x:Number, y:Number, amount:Number = 1 ) {
			var result:Selection = new Selection( this.parent );
			result.copy( this.selected );

			if( amount <= 0 ) {
				return result;
			}

			x = this.world2x( x );
			y = this.world2y( y );

			var stack:Array = [{x:x,y:y}];

			while( stack.length ) {
				var current:Object = stack.pop();
				x = current.x;
				y = current.y;

				if( result.selected[x][y] == 0 ) {
					result.selected[x][y] = amount;

					if( y < this.parent.size.y-1 )	stack.push( {x:x, y:y + 1} );
					if( y > 0 ) 					stack.push( {x:x, y:y - 1} );
					if( x < this.parent.size.x-1 ) 	stack.push( {x:x + 1, y:y} );
					if( x > 0 ) 					stack.push( {x:x - 1, y:y} );
				}
			}

			return result;
		}


		// Ceil (unused?) -----------------------------------------------------------------------------------
		public function ceil() {

			var result:Selection = new Selection( this.parent );

			for( var x = 0; x < this.parent.size.x; x++ ) {
				for( var y = 0; y < this.parent.size.y; y++ ) {
					result.selected[x][y] = this.selected[x][y] > 0 ? 1 : 0;
				}
			}

			return result;
		}


		// Glow (expand blur) -------------------------------------------------------------------------------
		public function glow( radius:Number ) {

			var temp:Selection = new Selection( this.parent );
			var result:Selection = new Selection( this.parent );

			radius = this.world2y( radius );

			// horz
			for( var x = 0; x < this.parent.size.x; x++ ) {
				for( var y = 0; y < this.parent.size.y; y++ ) {
					var sum:Number = 0;
					for( var kx:Number = -radius; kx <= radius; kx++ ) {
						var tx:Number = x + kx;
						if( tx < this.parent.size.x -1 && tx > 0) {
							sum += this.selected[tx][y];
						}
					}
					var gh:Number = sum / ( radius * 2 + 1 );
					temp.selected[x][y] = gh > this.selected[x][y] ? gh : this.selected[x][y];
				}
			}

			// vert
			for( x = 0; x < this.parent.size.x; x++ ) {
				for( y = 0; y < this.parent.size.y; y++ ) {
					sum = 0;
					for( var ky:Number = -radius; ky <= radius; ky++ ) {
						var ty:Number = y + ky;
						if( ty < this.parent.size.y -1 && ty > 0) {
							sum += temp.selected[x][ty];
						}
					}
					var gv:Number = sum / ( radius * 2 + 1 );
					result.selected[x][y] = gv > temp.selected[x][y] ? gv : temp.selected[x][y];
				}
			}

			return result;
		}

	}
}